/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
Date : 13/01/2013

CopyRight 2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <usart.h>

#include "PIC18F4431_config.h"

#include "delay.h"
#include "delay.c"

#define INTERRUPT_CONTROL
#include "init.h"
#include "init.c"

#include "CNC_settings.h"
#include "CNC_functions.h"
#include "CNC_menu.h"



//*******************************************
//    High Interrupt Priority Handlers
//*******************************************

__interrupt (high_priority) void ISR_cnc_signals (void) {

    // STEP Signal Handling
    if (CNC_STEP_FLAG == FLAG_SET) {

        if (step_status == ACTIVATED && enable_status == ACTIVATED) {

            missing_steps_counter++;
            
            if (missing_steps_counter == MAX_MISSING_STEPS)
                set_warning_status (WARNING_MISSING_STEPS);
        }          
        
        step_status = ACTIVATED;

        CNC_STEP_FLAG = FLAG_CLEAR;
    }
    
    // DIRECTION Signal Handling
    if (CNC_DIRECTION_FLAG == FLAG_SET ) {

        if (direction_status == ACTIVATED ) {
            direction_status = DEACTIVATED;
            INTCON2bits.INTEDG1 = RISING_EDGE;
        } else {
            direction_status = ACTIVATED;
            INTCON2bits.INTEDG1 = FALLING_EDGE;
        }

        CNC_DIRECTION_FLAG = FLAG_CLEAR;
    }

    // ENABLE Signal Handling
    if (CNC_ENABLE_FLAG == FLAG_SET ) {

        if (enable_status == ACTIVATED ) {
            stop_motor_rotation (MOTOR_ALL);
            enable_status = DEACTIVATED;
            phase_1_current = 0;
            phase_2_current = 0;
            INTCON2bits.INTEDG2 = RISING_EDGE;
        } else {
            enable_status = ACTIVATED;
            INTCON2bits.INTEDG2 = FALLING_EDGE;
        }     
        CNC_ENABLE_FLAG = FLAG_CLEAR;
    }

    // Border switches handling
    if (CNC_SWITCH_FLAG == 1 ) {

        // Check Switch 1
        if (AUX_SW_1 == DEACTIVATED) {
            switch_1_pressed_flag = ACTIVATED;            
        } 

        // Check Switch 2
        if (AUX_SW_2 == DEACTIVATED) {
            switch_2_pressed_flag = ACTIVATED;
        } 

        CNC_SWITCH_FLAG = 0;
    }
}


//*******************************************
//    Low Interrupt Priority Handlers
//*******************************************

__interrupt (low_priority) void ISR_cnc_measures (void) {

    // Variabile che conterr i dati ricevuti
    unsigned char data;
    static unsigned char led_tick = 0;
    static unsigned char status_led_tick = 0;
    static unsigned char voltage_tick = 0;
    static unsigned char system_check_tick = 0;
    static unsigned char temperature_tick = 0;
    static unsigned char previous_warning = 0;
    static unsigned char error_is_sent = 0;

    //********************************
    //     USART Interrupt Handler
    //********************************

    // Controllo che l'interrupt sia stato generato dall'USART
    if (PIR1bits.RCIF == 1 ) {

        // Read Databuffer
        read_data_RS485(&data);
      
        CNC_check_menu (data);

        PIR1bits.RCIF = 0;
    }


    // ********************************
    //     ADC Interrupt Handler
    // ********************************
    if (PIR1bits.ADIF == 1) {

	while (ADCON0bits.GO_DONE);

	ADC_buffer_reading = (int) ADRESH;
	ADC_buffer_reading = ADC_buffer_reading << 2;
	ADC_buffer_reading = ADC_buffer_reading + (int) (ADRESL >> 6);

        ADC_running = ADC_IS_FREE;

        PIR1bits.ADIF = 0;
    }


    //********************************
    //     Timer0 Interrupt Handler
    //********************************

    if (INTCONbits.TMR0IF == 1 ) {

        led_tick++;
        voltage_tick++;
        system_check_tick++;
        temperature_tick++;
        status_led_tick++;


        // Auxiliary Switch 1 Test
        if (switch_1_pressed_flag == ACTIVATED) {
            switch_1_filter_counter++;
            if (switch_1_filter_counter == AUX_SWITCH_FILTER_TICKS){
                set_error_status (ERROR_BOUDARY_SW_1);
            }
        }

        // Auxiliary Switch 2 Test
        if (switch_2_pressed_flag == ACTIVATED) {
            switch_2_filter_counter++;
            if (switch_2_filter_counter == AUX_SWITCH_FILTER_TICKS){
                set_error_status (ERROR_BOUDARY_SW_2);
            }
        }


        // Current Test
        if (phase_1_current > max_phase_current) {
            max_current_phase_1_counter++;            
            if (max_current_phase_1_counter > MAX_CURRENT_FILTER_TICKS)
                set_error_status (ERROR_OVER_CURRENT_M_1);
        }

        if (phase_2_current > max_phase_current) {
            max_current_phase_2_counter++;
            if (max_current_phase_2_counter > MAX_CURRENT_FILTER_TICKS)
                set_error_status (ERROR_OVER_CURRENT_M_2);
        }

        if (error_status != 0) {
            ERROR_LED = ACTIVATED;
            set_emergency_stop_out ();
            stop_motor_rotation (MOTOR_ALL);
            
            //Error is sent only once.
            if ( error_is_sent == DEACTIVATED) {

                #ifdef CNC_MENU
                    write_const_string_RS485 (" Error : ");
                    write_int_ASCII_RS485 (error_status, 3);
                    send_return_RS485 ();
                #endif
                error_is_sent = ACTIVATED;
            }
        }

        if (warning_status != 0 && error_status == 0) {
            if (status_led_tick == 3)
                ERROR_LED = ACTIVATED;

            if (status_led_tick == 4)
                ERROR_LED = DEACTIVATED;     
        }


        if (led_tick == 1 ) {
            HEART_LED = DEACTIVATED;
        }
        

        if (led_tick == LED_TICKS) {
            HEART_LED = ACTIVATED;

            max_current_phase_1_counter = 0;
            max_current_phase_2_counter = 0;

//            write_int_ASCII_RS485 (phase_1_current, 4);
//            write_const_string_RS485 (" ");
//            write_int_ASCII_RS485 (phase_2_current, 4);
//            write_const_string_RS485 (" ");
//            write_int_ASCII_RS485 (system_voltage, 3);
//            write_const_string_RS485 (" ");
//            write_int_ASCII_RS485 (system_temperature, 3);
//            send_return_RS485 ();

            if (warning_status != 0 ) {
                //I send warning only on delta
                
                if (warning_status != previous_warning) {

                    #ifdef CNC_MENU
                        write_const_string_RS485 (" Warning : ");
                        write_int_ASCII_RS485 (warning_status, 3);
                        send_return_RS485 ();
                    #endif
                    previous_warning = warning_status;
                }
            }

            status_led_tick = 0;
            led_tick = 0;
        }

        
        if (voltage_tick == VOLTAGE_TICKS) {
            // Request a Voltage Measure
            voltage_measure_request = ACTIVATED;
            voltage_tick = 0;
        }

        if (temperature_tick == TEMPERATURE_TICKS) {
            // Request a Temeprature Measure
            temperature_measure_request = ACTIVATED;
            temperature_tick = 0;
        }

        if (system_check_tick == SYSTEM_MONITOR_TICKS) {
            // Request a System Check
            sytem_monitor_request = ACTIVATED;
            system_check_tick = 0;
        }
                
        // Clear Timer Flag
        INTCONbits.TMR0IF = 0;
    }
}

//************************************************************
// Main Program
//************************************************************

int main (void){

    board_initialization ();

    CNC_system_initialization ();

    // State machine Execution

    while (1) {
        
        if (RCSTAbits.FERR == ACTIVATED) {
            set_warning_status (WARNING_COMMUNICATION);            
        }

        //Clear errors
        if (RCSTAbits.FERR == ACTIVATED || RCSTAbits.OERR == ACTIVATED) {
            RCSTAbits.SPEN = DEACTIVATED;
            initialize_RS485 (BAUD_RATE_115200,RX_INTERRUPT_ON, TX_INTERRUPT_OFF);
            //set_warning_status (WARNING_COMMUNICATION);
        }
        
        if (enable_status == ACTIVATED ){

           if (step_status == ACTIVATED && direction_status == ACTIVATED) {
                
                step_motor_clockwise (0);
                step_status = DEACTIVATED;
            }

            if (step_status == ACTIVATED && direction_status == DEACTIVATED) {

                step_motor_anticlockwise (0);
                step_status = DEACTIVATED;
            }    
        }

        //****************************************
        // Analog Measures Voltage - Temperature
        //****************************************
        if (voltage_measure_request == ACTIVATED) {
            CNC_state_voltage_measure ();
        }

        if (temperature_measure_request == ACTIVATED) {
            CNC_state_temperature_measure ();
        }

        if (sytem_monitor_request == ACTIVATED) {
            CNC_state_system_monitor ();
        }

        //****************************************
        // Analog Measures: Voltage, Current and Temperature
        //****************************************

        if (ADC_running == ADC_IS_FREE) {

            switch (ADC_State) {

                case ADC_PHASE_1_TEST:
                                        CNC_state_phase_1_current_read ();
                                        break;
                case ADC_PHASE_2_TEST:
                                        CNC_state_phase_2_current_read ();
                                        break;
                case ADC_VIN_TEST:
                                        CNC_state_Vin_read ();
                                        break;
                case ADC_TEMP_TEST:
                                        CNC_state_temperature_read ();
                                        break;
                default:
                                        break;
             }
        }
    }
}



